# Sample Draft script
#
# Convert an image sequence to an H.264 movie, and a half-resolution H.264 proxy movie.
# The movies include a slate and annotations.
#
# In order to use Deadline.DeadlineConnect to fetch job properties, follow the setup instructions at:
#    https://docs.thinkboxsoftware.com/products/deadline/10.1/1_User%20Manual/manual/standalone-python.html
# then replace PulseName and PulsePortNumber with the correct values.

from __future__ import absolute_import, print_function
import sys
import os
import datetime
import copy

import Draft
from DraftParamParser import FrameRangeToFrames, ParseCommandLine, ReplaceFilenameHashesWithNumber

# The argument name/types we're expecting from the command line arguments
expectedTypes = dict()
expectedTypes['frameList'] = '<string>'
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'
expectedTypes['username'] = '<string>'
expectedTypes['entity'] = '<string>'
expectedTypes['version'] = '<string>'

# Parse the command line arguments
params = ParseCommandLine(expectedTypes, sys.argv)

# Path to the Deadline Repository root
deadlineRepo = params['deadlineRepository']

# TODO: CHANGE ME! Path to an image containing the background of the slate frame
slateFrame = params['deadlineRepository'] + '/Draft/Samples/slateFrame.png'

inFilePattern = params['inFile']
frames = FrameRangeToFrames(params['frameList'])
(outBase, outExt) = os.path.splitext(params['outFile'])

# not a huge deal if we can't connect to the repo, we'll just be missing some info
try:
    import Deadline.DeadlineConnect as Connect

    # TODO: CHANGE ME!
    # 'PulseName' is the DNS name or IP address of the machine currently running Pulse (and Web Service)
    #   and configured as the Pulse machine in the [Repository Options].
    # 'PulsePortNumber' is the Pulse Web Service Port Number as configured in the [Repository Options] -
    #   [Pulse Settings] - [Web Service] tab. By default it is: 8080.
    connectionObject = Connect.DeadlineCon('PulseName', PulsePortNumber)
    jobId = params['deadlineJobID']
    job = connectionObject.Jobs.GetJob(jobId)
    jobParams = job['Props']
except Exception as ex:
    print("Error:", ex)
    jobParams = {}

outWidth = 1920
outHeight = 1080
proxyScale = 0.5
proxyWidth = int(proxyScale * outWidth)
proxyHeight = int(proxyScale * outHeight)
slateFrames = 24

# Build up the encoders
encoder = Draft.VideoEncoder(outBase + outExt, 24, outWidth, outHeight, codec="H264")
proxyEncoder = Draft.VideoEncoder(outBase + "-proxy" + outExt, 24, proxyWidth, proxyHeight, codec="H264")

# Annotation info used for burn ins
annotationInfo = Draft.AnnotationInfo()
annotationInfo.PointSize = int(outHeight * 0.045)
annotationInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)

# prep the Slate Frame
try:
    slate = Draft.Image.ReadFromFile(slateFrame)
except:
    slate = Draft.Image.CreateImage(outWidth, outHeight)
    slate.SetToColor(Draft.ColorRGBA(0.0, 0.0, 0.0, 1.0))

slate.Resize(outWidth, outHeight)

# sets up the text on the slate frame
slateAnnotations = [
    ("SHOW", jobParams.get('Ex1', None)),  # This line is skipped if there is not Ex1
    ("EPISODE", params.get('episode', None)),  # This line is skipped if 'episode' isn't in the extra args
    ("SHOT", params['entity']),
    ("FRAMES", params['frameList']),
    ("HANDLES", params.get('handles', None)),  # This line is skipped if 'handles' isn't in the extra args
    ("VERSION", params['version']),
    ("", ''),
    ("", ''),
    ("ARTIST", params['username']),
    ("DATE/TIME", datetime.datetime.now().strftime("%m/%d/%Y %I:%M %p"))
]

# comp the annotations over top the slate frame
skipLines = 0
for i in range(0, len(slateAnnotations)):
    annotationTuple = slateAnnotations[i]

    if annotationTuple[1] is None:
        skipLines += 1
        continue

    lineNum = i - skipLines
    if annotationTuple[0] != "":
        annotation = Draft.Image.CreateAnnotation(slateAnnotations[i][0] + ": ", annotationInfo)
        slate.CompositeWithPositionAndAnchor(annotation, 0.45, 0.7 - (lineNum * 0.06), Draft.Anchor.SouthEast,
                                             Draft.CompositeOperator.OverCompositeOp)

    if annotationTuple[1] != "":
        annotation = Draft.Image.CreateAnnotation(slateAnnotations[i][1], annotationInfo)
        slate.CompositeWithPositionAndAnchor(annotation, 0.46, 0.7 - (lineNum * 0.06), Draft.Anchor.SouthWest,
                                             Draft.CompositeOperator.OverCompositeOp)

proxySlate = copy.deepcopy(slate)
proxySlate.Resize(proxyWidth, proxyHeight)

progressCounter = 0
totalFrames = slateFrames + len(frames)

# encode the slate frames at the start of the video
print("Encoding Slate Frames...")
for i in range(0, slateFrames):
    encoder.EncodeNextFrame(slate)
    proxyEncoder.EncodeNextFrame(proxySlate)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

studioAnnotation = Draft.Image.CreateAnnotation("Thinkbox Software Inc.", annotationInfo)
entityAnnotation = Draft.Image.CreateAnnotation(
    "%s    %s" % (params['entity'], datetime.datetime.now().strftime("%m/%d/%Y")), annotationInfo)
annotationInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 1.0)

# Main encoding loop
for frameNumber in frames:
    print("Processing Frame: %d..." % frameNumber)

    inFile = ReplaceFilenameHashesWithNumber(inFilePattern, frameNumber)
    bgFrame = Draft.Image.ReadFromFile(inFile)

    bgFrame.Resize(outWidth, outHeight)
    bgFrame.SetChannel('A', 1.0)

    # Do the frame burnins
    framesAnnotation = Draft.Image.CreateAnnotation(str(frameNumber), annotationInfo)
    bgFrame.CompositeWithAnchor(studioAnnotation, Draft.Anchor.NorthWest, Draft.CompositeOperator.OverCompositeOp)
    bgFrame.CompositeWithAnchor(entityAnnotation, Draft.Anchor.SouthWest, Draft.CompositeOperator.OverCompositeOp)
    bgFrame.CompositeWithAnchor(framesAnnotation, Draft.Anchor.SouthEast, Draft.CompositeOperator.OverCompositeOp)

    encoder.EncodeNextFrame(bgFrame)

    bgFrame.Resize(proxyWidth, proxyHeight)

    proxyEncoder.EncodeNextFrame(bgFrame)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

# Finalize the encoding process
encoder.FinalizeEncoding()
proxyEncoder.FinalizeEncoding()
