# Sample script that applies an ASC CDL LUT to an image sequence, which are then encoded to a video.
#
# Sample additional args:
#    asccdl='//path/to/asc_cdl_file.cdl'
from __future__ import absolute_import, print_function
import sys

import Draft
from DraftASCCDLReader import ReadASCCDL  # ****** Function to read ASC CDL from file and return a Draft.LUT
from DraftParamParser import FrameRangeToFrames, ParseCommandLine, ReplaceFilenameHashesWithNumber

# The argument name/types we're expecting from the command line arguments or Deadline.
expectedTypes = dict()
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'
expectedTypes['frameList'] = '<string>'

# uncomment this line if you want the asccdl parameter to be mandatory rather than optional
# expectedTypes['asccdl'] = '<string>'

# params now contains a dictionary of the parameters initialized to values from the command line arguments.
params = ParseCommandLine(expectedTypes, sys.argv)
inFilePattern = params['inFile']  # for example frame_name_###.ext, where ### represents a three digit frame number.
frames = FrameRangeToFrames(params['frameList'])  # Get a list of the individual frames we are to process

lut = None
if 'asccdl' in params:
    lut = ReadASCCDL(params['asccdl'])  # Get the ASC CDL, if a parameter named 'asccdl' is in the parameter list

# Initialize the video encoder.
frame = Draft.Image.ReadFromFile(
    ReplaceFilenameHashesWithNumber(inFilePattern, frames[0]))  # use the first frame to determine video size
encoder = Draft.VideoEncoder(params['outFile'], 24, frame.width, frame.height)

# Process each of the frames in the list of frames (including the first, which hasn't yet been added to the encoder).
progressCounter = 0
for currFrame in frames:
    # Read in the frame.
    currFile = ReplaceFilenameHashesWithNumber(inFilePattern, currFrame)
    frame = Draft.Image.ReadFromFile(currFile)

    if lut is not None:
        lut.Apply(frame)  # ****** apply the ASC CDL LUT to the frame

    # Add the frame to the encoder.
    encoder.EncodeNextFrame(frame)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // len(frames)
    print("Progress: %i%%" % progress)

# Finalize and save the resulting video.
encoder.FinalizeEncoding()
