from __future__ import absolute_import, print_function
import six

try:
    # Try importing the standard libraries used by this script.  If this fails, the problem is bigger than Draft.
    import sys
    import imp
    import os
except ImportError as ie:
    print('\n*** Importing a standard python library failed, error message: ***')
    print(ie)
    print('There is a problem with the installation of python.  Please fix that, then try running this script again.')
    exit(1)

if '-h' in sys.argv:
    print('\nOptions and arguments:')
    print('-h : print this help message and exit')
    print('-v : verbose (print diagnostic information whether or not problems are found)')
    print('\nThis python script is for debugging the installation of the Draft python module.')
    exit(0)


def looks_like_Draft_path_candidate(path):
    if sys.platform == 'win32':
        (unc, path) = os.path.splitunc(path)  # discard the UNC portion of the path on Windows if it's there
    dir = os.path.dirname(path)
    dir_lc = dir.lower()
    is_candidate = ('deadline' in dir_lc) and ('draft' in dir_lc)
    return is_candidate


def find_in_files(prefix, ext, pattern, files):
    filename = ''
    for file in files:
        if file.startswith(prefix):
            if ext in file:
                filename = file
            else:
                print(file, 'has wrong extension, was expecting', ext)
            break
    if filename == '':
        print('missing Draft file:', pattern)
    return filename

# set to True if we've reached a dead-end, and want to print a bunch of
# diagnostics information that might be helpful in debugging the problem
print_diagnositcs = False

verbose = False
if '-v' in sys.argv:
    verbose = True  # print the diagnostics information regardless

try:
    print("\n****** Attempting to import Draft. ******")
    import Draft

    print("*** Draft imported successfully!  You're good to go! ***")
    # Draft versions older than 1.0 don't print the version number automatically
    print("Draft version", Draft.LibraryInfo.Version())
    print("\nNote:  If you are getting errors when running a Draft script, "
          "look for errors in the script, or with the input to the script.\n")
    # raise ImportError('DEBUG:  forcing execution of except clause')   # For script-debugging purposes only!
except ImportError as ide:
    print('*** "import Draft" failed, error message: ***')
    print(ide)

    try:
        print('\n****** Attempting to find Draft. ******')
        (file, path, descr) = imp.find_module("Draft")
        print('Draft found in:', path)

        try:
            # does the path look right?
            if not looks_like_Draft_path_candidate(path):
                print(
                    "*** WARNING:  Are you sure that's the correct path for the Draft module?  It doesn't appear to be in the usual place. ***")
                print_diagnositcs = True

            # does the directory contain the expected files? 
            print('\n****** Checking Draft directory contents. ******')
            dir = os.path.dirname(path)
            files = os.listdir(dir)

            # expected files:  platform independent tests
            pi_files = ['colors.xml', 'DraftParamParser.py', 'SourceSansPro-Regular.otf', 'type.xml', 'Version']
            for file in pi_files:
                if file not in files:
                    print('missing Draft file:', file)

            # expected files:  platform specific tests
            if sys.platform.startswith('linux'):
                ps_file_pairs = [('Draft.', '.so', 'Draft.so'), ('libavcodec.', '.so', 'libavcodec.so.*'),
                                 ('libavformat.', '.so', 'libavformat.so.*'), ('libavutil.', '.so', 'libavutil.so.*'),
                                 ('libswscale.', '.so', 'libswscale.so.*'), ('libtbb.', '.so', 'libtbb.so.*'),
                                 ('libx264.', '.so', 'libx264.so.*')]
                for (prefix, ext, pattern) in ps_file_pairs:
                    filename = find_in_files(prefix, ext, pattern, files)
                    # if filename != '':
                    # check 32/64 bit?  other test?
            elif sys.platform == 'win32':
                ps_file_pairs = [('Draft.', '.pyd', 'Draft.pyd'), ('avcodec-', '.dll', 'avcodec-*.dll'),
                                 ('avformat-', '.dll', 'avformat-*.dll'), ('avutil-', '.dll', 'avutil-*.dll'),
                                 ('boost_python-', '.dll', 'boost_python-*.dll'), ('swscale-', '.dll', 'swscale-*.dll'),
                                 ('tbb.', '.dll', 'tbb.dll'), ('libx264-', '.dll', 'libx264-*.dll')]
                for (prefix, ext, pattern) in ps_file_pairs:
                    filename = find_in_files(prefix, ext, pattern, files)
                    # if filename != '':
                    # check 32/64 bit?  other test?
            elif sys.platform == 'darwin':
                ps_file_pairs = [('Draft.', '.so', 'Draft.so'), ('libavcodec.', '.dylib', 'libavcodec.*.dylib'),
                                 ('libavformat.', '.dylib', 'libavformat.*.dylib'),
                                 ('libavutil.', '.dylib', 'libavutil.*.dylib'),
                                 ('libswscale.', '.dylib', 'libswscale.*.dylib'), ('libtbb.', '.dylib', 'libtbb.dylib'),
                                 ('libx264.', '.dylib', 'libx264.*.dylib')]
                for (prefix, ext, pattern) in ps_file_pairs:
                    filename = find_in_files(prefix, ext, pattern, files)
            else:
                print('*** DEBUG:  unexpected platform found! ***')
                print('Please inform Thinkbox Support that the following platform was '
                      'encountered by check_Draft_install.py:', sys.platform)
                print_diagnositcs = True

            print('*** finished checking directory contents ***')
        except Exception as unk:
            print('\n*** Something went wrong while trying to check the directory path and contents, '
                  'error message: ***')
            print(unk)
            print_diagnositcs = True

    except Exception as fme:
        print('\n*** Problem finding Draft module, error message: ***')
        print(fme)
        print_diagnositcs = True

        # does sys.path have something that looks like it could be the Draft path?
        found_potential_path = False
        for path in sys.path:
            if looks_like_Draft_path_candidate(path):
                found_potential_path = True
                break

        if not found_potential_path:
            print(
                '*** Nothing in sys.path looks like it could be the Draft path.  '
                'The problem could be with PYTHONPATH. ***')

if print_diagnositcs or verbose:
    print('\n****** printing diagnostic information ******')
    print("\nsys.version =", sys.version)
    print("sys.version_info =", sys.version_info)
    print("sys.api_version =", sys.api_version)
    print("sys.platform =", sys.platform)
    print("sys.prefix =", sys.prefix)
    print("sys.meta_path =", sys.meta_path)

    print("\nsys.path:")
    for path in sys.path:
        print(path)

    print("\nos.environ:")
    for key, value in six.iteritems(os.environ):
        print("%25s = %s" % (key, value))
