# Sample Draft script
#
# Convert an image sequence to an MJPEG movie.
# The movie includes a slate, annotations, and a translucent frame.

from __future__ import absolute_import, print_function
import sys
import datetime
import copy

from Draft import Anchor, AnnotationInfo, ColorRGBA, CompositeOperator, Image, VideoEncoder
from DraftParamParser import ParseCommandLine, ReplaceFilenameHashesWithNumber

scale = 0.5
overlayScale = 0.15

expectedTypes = {}
expectedTypes['username'] = '<string>'
expectedTypes['entity'] = '<string>'
expectedTypes['version'] = '<string>'
expectedTypes['startFrame'] = '<int>'
expectedTypes['endFrame'] = '<int>'
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'

params = ParseCommandLine(expectedTypes, sys.argv)

# TODO: CHANGE ME! Path to an image containing the logo image
logoFile = params['deadlineRepository'] + '/Draft/Samples/logoSmall.png'

inFilePattern = params['inFile']
startFrame = int(params['startFrame'])
endFrame = int(params['endFrame'])

# pull the output width height from an input frame
frameNumber = startFrame
inFile = ReplaceFilenameHashesWithNumber(inFilePattern, frameNumber)
inFrame = Image.ReadFromFile(inFile)

# apply the scale
width = int(inFrame.width * scale)
height = int(inFrame.height * scale)

# create an oversized background frame to composit everything onto
overlayStrip = Image.CreateImage(width, int(height * overlayScale))
overlayStrip.SetToColor(ColorRGBA(0.0, 0.0, 0.0, 0.2))
overlay = Image.CreateImage(width, height)
overlay.SetToColor(ColorRGBA(0.0, 0.0, 0.0, 0.0))
overlay.CompositeWithPositionAndAnchor(overlayStrip, 0.0, 0.0, Anchor.SouthWest, CompositeOperator.OverCompositeOp)
overlay.CompositeWithPositionAndAnchor(overlayStrip, 0.0, 1.0, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

bgColor = ColorRGBA(0.0, 0.0, 0.0, 1.0)
origBgFrame = Image.CreateImage(width, height)
origBgFrame.SetToColor(bgColor)

# make a video encoder
slateFrames = 24
fps = 24
quality = 85
codec = "MJPEG"
videoEncoder = VideoEncoder(params['outFile'], fps, origBgFrame.width, origBgFrame.height, quality=quality, codec=codec)

# encode the slate frames
slateBgFrame = copy.deepcopy(origBgFrame)
annotationInfo = AnnotationInfo()
annotationInfo.FontWeight = 100
annotationInfo.Color = ColorRGBA(0.0, 0.85, 0.85, 1.0)
annotationInfo.DrawShadow = True
annotationInfo.ShadowColor = ColorRGBA(0.0, 0.0, 0.85, 1.0)

annotationInfo.PointSize = int(height * 0.11)
annotation = Image.CreateAnnotation("DRAFT DEMO ", annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.5, 0.9, Anchor.Center, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.11)
annotation = Image.CreateAnnotation("SLATING/TEMPLATING ", annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.5, 0.8, Anchor.Center, CompositeOperator.OverCompositeOp)

annotationInfo.DrawShadow = False
annotationInfo.Color = ColorRGBA(0.85, 0.85, 0.85, 1.0)

annotationInfo.PointSize = int(height * 0.06)
annotation = Image.CreateAnnotation(params['username'], annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.1, 0.65, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.06)
annotation = Image.CreateAnnotation(params['entity'], annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.1, 0.58, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.06)
annotation = Image.CreateAnnotation(params['version'], annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.1, 0.51, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.05)
annotation = Image.CreateAnnotation("Frame Range:  " + str(startFrame) + " - " + str(endFrame), annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.15, 0.45, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.05)
annotation = Image.CreateAnnotation("Resolution: " + str(inFrame.width) + " x " + str(inFrame.height), annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.15, 0.40, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.05)
annotationInfo.Color = ColorRGBA(0.85, 0.85, 0.85, 1.0)
annotation = Image.CreateAnnotation(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.025, 0.08, Anchor.SouthWest,
                                            CompositeOperator.OverCompositeOp)

annotationInfo.PointSize = int(height * 0.035)
annotationInfo.Color = ColorRGBA(0.85, 0.85, 0.85, 1.0)
annotation = Image.CreateAnnotation(inFile, annotationInfo)
slateBgFrame.CompositeWithPositionAndAnchor(annotation, 0.025, 0.025, Anchor.SouthWest,
                                            CompositeOperator.OverCompositeOp)

progressCounter = 0
totalFrames = slateFrames + (endFrame - startFrame + 1)

for frameNumber in range(0, slateFrames):
    videoEncoder.EncodeNextFrame(slateBgFrame)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

# add a logo to the frame if you can find one
try:
    logo = Image.ReadFromFile(logoFile)
    logoHeight = int(overlayScale * origBgFrame.height)
    logoWidth = int(logo.width * logoHeight // logo.height)
    logo.Resize(logoWidth, logoHeight)
except:
    logo = None

# encode the video frames
for frameNumber in range(startFrame, endFrame + 1):

    print("Processing frame: " + str(frameNumber))
    sys.stdout.flush()

    inFile = ReplaceFilenameHashesWithNumber(inFilePattern, frameNumber)
    bgFrame = Image.ReadFromFile(inFile)
    bgFrame.Resize(width, height)
    bgFrame.Composite(overlay, 0.0, 0.0, CompositeOperator.OverCompositeOp)

    if logo is not None:
        bgFrame.CompositeWithPositionAndAnchor(logo, 1.0, 0.0, Anchor.SouthEast, CompositeOperator.OverCompositeOp)

    annotationInfo = AnnotationInfo()

    annotationInfo.Color = ColorRGBA(0.85, 0.85, 0.85, 1.0)
    annotationInfo.DrawShadow = True
    annotationInfo.ShadowColor = ColorRGBA(0.1, 0.1, 0.1, 1.0)

    annotationInfo.PointSize = int(height * 0.06)
    annotation = Image.CreateAnnotation(params['username'], annotationInfo)
    bgFrame.CompositeWithPositionAndAnchor(annotation, 0.025, 0.97, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

    annotationInfo.PointSize = int(height * 0.045)
    annotation = Image.CreateAnnotation(datetime.datetime.now().strftime("%Y-%m-%d"), annotationInfo)
    bgFrame.CompositeWithPositionAndAnchor(annotation, 0.025, 0.91, Anchor.NorthWest, CompositeOperator.OverCompositeOp)

    annotationInfo.PointSize = int(height * 0.05)
    annotation = Image.CreateAnnotation(params['entity'], annotationInfo)
    bgFrame.CompositeWithPositionAndAnchor(annotation, 0.03, 0.08, Anchor.SouthWest, CompositeOperator.OverCompositeOp)

    annotationInfo.PointSize = int(height * 0.045)
    annotation = Image.CreateAnnotation(params['version'], annotationInfo)
    bgFrame.CompositeWithPositionAndAnchor(annotation, 0.03, 0.025, Anchor.SouthWest, CompositeOperator.OverCompositeOp)

    annotationInfo.PointSize = int(height * 0.08)
    annotation = Image.CreateAnnotation(str(frameNumber), annotationInfo)
    bgFrame.CompositeWithPositionAndAnchor(annotation, 0.95, 0.95, Anchor.NorthEast, CompositeOperator.OverCompositeOp)

    videoEncoder.EncodeNextFrame(bgFrame)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

# finalize the encoding
videoEncoder.FinalizeEncoding()
