# Sample Draft script
#
# Convert an image sequence to a MJPEG movie.
# The movie includes a slate, annotations, and a watermark.

from __future__ import absolute_import, print_function
import sys
import datetime
import Draft
from DraftParamParser import ParseCommandLine, ReplaceFilenameHashesWithNumber

# Retrieve command line parameters using Draft's param parsing tool.
# Make sure to read the comments in DraftParamParser.py for more info!
expectedTypes = {}
expectedTypes['username'] = '<string>'
expectedTypes['entity'] = '<string>'
expectedTypes['version'] = '<string>'
expectedTypes['startFrame'] = '<int>'
expectedTypes['endFrame'] = '<int>'
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'

params = ParseCommandLine(expectedTypes, sys.argv)

# reads in the first frame to get the width/height of the sequence
firstFrame = Draft.Image.ReadFromFile(ReplaceFilenameHashesWithNumber(params['inFile'], params['startFrame']))
width = firstFrame.width
height = firstFrame.height

# Set up variables for video encoding
fps = 24
quality = 85
codec = "MJPEG"
videoEncoder = Draft.VideoEncoder(params['outFile'], fps, width, height, quality=quality, codec=codec)

# Creates a slate that will be shown at the start of the video
slateFrames = 24
slate = Draft.Image.CreateImage(width, height)
slate.SetToColor(Draft.ColorRGBA(0.0, 0.0, 0.0, 1.0))

print("Creating intro slate")
sys.stdout.flush()

# Adds textual annotations to the Slate created above
textInfo = Draft.AnnotationInfo()
textInfo.Padding = 0.2
textInfo.PointSize = 34
textInfo.Color = Draft.ColorRGBA(0.862745, 0.913726, 1.0, 1.0)
textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
textInfo.DrawShadow = True
textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
textImage = Draft.Image.CreateAnnotation("DEADLINE DRAFT WATERMARK DEMO", textInfo)
slate.CompositeWithPositionAndAnchor(textImage, 0.0375, 0.9, Draft.Anchor.SouthWest,
                                     Draft.CompositeOperator.OverCompositeOp)
textInfo = Draft.AnnotationInfo()
textInfo.Padding = 0.2
textInfo.PointSize = 24
textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
textInfo.DrawShadow = True
textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
textImage = Draft.Image.CreateAnnotation(params['inFile'], textInfo)
slate.CompositeWithPositionAndAnchor(textImage, 0.007, 0.005, Draft.Anchor.SouthWest,
                                     Draft.CompositeOperator.OverCompositeOp)
textInfo = Draft.AnnotationInfo()
textInfo.Padding = 0.2
textInfo.PointSize = 24
textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
textInfo.DrawShadow = True
textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
textImage = Draft.Image.CreateAnnotation(params['username'], textInfo)
slate.CompositeWithPositionAndAnchor(textImage, 0.0375, 0.578334, Draft.Anchor.SouthWest,
                                     Draft.CompositeOperator.OverCompositeOp)
textInfo = Draft.AnnotationInfo()
textInfo.Padding = 0.2
textInfo.PointSize = 24
textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
textInfo.DrawShadow = True
textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
textImage = Draft.Image.CreateAnnotation(params['entity'], textInfo)
slate.CompositeWithPositionAndAnchor(textImage, 0.0375, 0.515001, Draft.Anchor.SouthWest,
                                     Draft.CompositeOperator.OverCompositeOp)
textInfo = Draft.AnnotationInfo()
textInfo.Padding = 0.2
textInfo.PointSize = 24
textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 0.992157, 1.0)
textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
textInfo.DrawShadow = True
textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
textImage = Draft.Image.CreateAnnotation(params['version'], textInfo)
slate.CompositeWithPositionAndAnchor(textImage, 0.03875, 0.456667, Draft.Anchor.SouthWest,
                                     Draft.CompositeOperator.OverCompositeOp)

progressCounter = 0
totalFrames = slateFrames + (params['endFrame'] - params['startFrame'] + 1)

# Encodes the slate frames
for i in range(0, slateFrames):
    videoEncoder.EncodeNextFrame(slate)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

# Loop start/end is dictated by the command line input
theLoopStart = int(params['startFrame'])
theLoopEnd = int(params['endFrame'])

# CHANGE ME! Path to an image containing the watermark image
compositeImage0 = Draft.Image.ReadFromFile(params['deadlineRepository'] + '/Draft/Samples/logo.png')
alphaImage = Draft.Image.CreateImage(compositeImage0.width, compositeImage0.height)
alphaImage.SetChannel('A', 0.25)
alphaImage.Composite(compositeImage0, 0, 0, Draft.CompositeOperator.InCompositeOp)
compositeImage0.Composite(alphaImage, 0, 0, Draft.CompositeOperator.CopyOpacityCompositeOp)

# Goes through the input frames, adding on the watermark & some annotations, then encoding them into the final video
for aFrame in range(theLoopStart, theLoopEnd):
    print("processing frame " + str(aFrame))
    sys.stdout.flush()

    # Replaces the '#' symbols in the filename pattern to get this frame's image
    inFilePattern = params['inFile']
    inFilePattern = ReplaceFilenameHashesWithNumber(inFilePattern, aFrame)

    # Read in the image file
    slate = Draft.Image.ReadFromFile(inFilePattern)
    slate.SetChannel('A', 1.0)

    # Add the watermark
    slate.CompositeWithPositionAndAnchor(compositeImage0, 0.00875, -0.00833321, Draft.Anchor.SouthWest,
                                         Draft.CompositeOperator.OverCompositeOp)

    # Add some annotations
    textInfo = Draft.AnnotationInfo()
    textInfo.Padding = 0.2
    textInfo.PointSize = 36
    textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
    textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
    textInfo.DrawShadow = True
    textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
    textImage = Draft.Image.CreateAnnotation(str(aFrame), textInfo)
    slate.CompositeWithPositionAndAnchor(textImage, 0.8825, 0.9, Draft.Anchor.SouthWest,
                                         Draft.CompositeOperator.OverCompositeOp)
    textInfo = Draft.AnnotationInfo()
    textInfo.Padding = 0.2
    textInfo.PointSize = 24
    textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
    textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
    textInfo.DrawShadow = True
    textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 1.0)
    textImage = Draft.Image.CreateAnnotation(datetime.datetime.now().strftime('Date: %m/%d/%Y'), textInfo)
    slate.CompositeWithPositionAndAnchor(textImage, 0.0125, 0.933333, Draft.Anchor.SouthWest,
                                         Draft.CompositeOperator.OverCompositeOp)
    textInfo = Draft.AnnotationInfo()
    textInfo.Padding = 0.2
    textInfo.PointSize = 24
    textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
    textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
    textInfo.DrawShadow = True
    textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.392157, 1.0)
    textImage = Draft.Image.CreateAnnotation(inFilePattern, textInfo)
    slate.CompositeWithPositionAndAnchor(textImage, 0.0065, 0.00466663, Draft.Anchor.SouthWest,
                                         Draft.CompositeOperator.OverCompositeOp)
    textInfo = Draft.AnnotationInfo()
    textInfo.Padding = 0.2
    textInfo.PointSize = 24
    textInfo.Color = Draft.ColorRGBA(1.0, 1.0, 1.0, 1.0)
    textInfo.BackgroundColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 0.0)
    textInfo.DrawShadow = True
    textInfo.ShadowColor = Draft.ColorRGBA(0.0, 0.0, 0.0, 1.0)
    textImage = Draft.Image.CreateAnnotation(params['username'], textInfo)
    slate.CompositeWithPositionAndAnchor(textImage, 0.0125, 0.866667, Draft.Anchor.SouthWest,
                                         Draft.CompositeOperator.OverCompositeOp)

    # encodes the video frame for this image
    videoEncoder.EncodeNextFrame(slate)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // totalFrames
    print("Progress: %i%%" % progress)

# finalizes the video encoding
videoEncoder.FinalizeEncoding()
