# Sample Draft script:  Basic Quicktime
# Convert linear frames to a QT MJPEG whose height is specified in a command line argument 
#
#
# When submitting this script to Deadline, the following Draft Options should be set:
#   Input File:  Select one of the frames in the sequence that will be processed.
#   Output File:  Leave it as Deadline auto-fills, or select an alternate location/name for the movie file.
#   Frame List:  Leave it as Deadline auto-fills, or specify a subset of the frames.
#   Additional Args:  height=# (where # is the desired height in pixels)
#
#
# If running this script from the command line, 
#   infile:  The pattern that the input files follow, for example frame_name_###.ext, where ### represents a three digit
#            frame number, due to there being three hash marks.
#   outfile:  The (path and) filename of the movie file.
#   frameList:  The list of frames to be processed.
#   height:  The height (in pixels) of the output movie.
#
# Sample command line:
#   python encode_to_MOV_MJPEG_param.py inFile=path\name_####.jpg outFile=path\movie_name.mov frameList=0-100 height=480

from __future__ import absolute_import, print_function
import Draft
import sys  # To access commmand line arguments.  Deadline sends script parameters as command line arguments.
from DraftParamParser import FrameRangeToFrames, ParseCommandLine, ReplaceFilenameHashesWithNumber

# Constants we will use to specify how we want our movie encoded.
CODEC = "MJPEG"  # Use the MJPEG codec

# The argument name/types we're expecting from the command line arguments or Deadline.
expectedTypes = dict()
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'
expectedTypes['frameList'] = '<string>'
expectedTypes['height'] = '<int>'

# params now contains a dictionary of the parameters initialized to values from the command line arguments.
params = ParseCommandLine(expectedTypes, sys.argv)
inFilePattern = params['inFile']  # for example frame_name_###.ext, where ### represents a three digit frame number.
frames = FrameRangeToFrames(params['frameList'])  # Get a list of the individual frames we are to process
OUTPUT_HEIGHT = params['height']  # Get the desired output height from the input parameters

# Get the first frame to figure out our input width/height.  (Assumes all frames have the same width & height.)
filename = ReplaceFilenameHashesWithNumber(inFilePattern, frames[0])  # make a filename with the first frame number
firstFrame = Draft.Image.ReadFromFile(filename)  # Read the frame data into a Draft image.

inputWidth = firstFrame.width
inputHeight = firstFrame.height

# Calculate our scaling and output width based on input and desired height (maintain original aspect ratio).
scaling = float(OUTPUT_HEIGHT) / inputHeight  # force floating-point division
outputWidth = int(round(inputWidth * scaling))

# Initialize the video encoder.
encoder = Draft.VideoEncoder(params['outFile'], width=outputWidth, height=OUTPUT_HEIGHT, codec=CODEC)

# Process each of the frames in the list of frames (including the first, which hasn't yet been added to the encoder).
progressCounter = 0
for currFrame in frames:
    # Read in the frame.
    currFile = ReplaceFilenameHashesWithNumber(inFilePattern, currFrame)
    frame = Draft.Image.ReadFromFile(currFile)

    # Resize the frame to match the output dimensions.
    frame.Resize(outputWidth, OUTPUT_HEIGHT, type='fill')

    # Add the frame to the encoder.
    encoder.EncodeNextFrame(frame)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // len(frames)
    print("Progress: %i%%" % progress)

# Finalize and save the resulting video.
encoder.FinalizeEncoding()
