# Sample Draft script:  Resize Set
# Resize image to 640 width on a 640x480 background.
#
#
# When submitting this script to Deadline, the following Draft Options should be set:
#   Input File:  Select one of the frames in the sequence that will be processed.
#   Output File:  Leave it as Deadline auto-fills (correcting the file extension), or select an alternate location
#                 (& optional file name pattern) for the output files.
#   Frame List:  Leave it as Deadline auto-fills, or specify a subset of the frames.
#
#
# If running this script from the command line, 
#   infile:  The pattern that the input files follow, for example frame_name_###.ext, where ### represents a three digit
#            frame number, due to there being three hash marks.
#   outfile:  The directory (& optional file name pattern) to save the output files.
#   frameList:  The list of frames to be processed.
#
# Sample command line:
#   python resize_image_640x480_fit_width.py inFile=path\frame_name_####.ext outFile=path frameList=0-100
from __future__ import absolute_import, print_function
import Draft
import sys  # To access commmand line arguments.  Deadline sends script parameters as command line arguments.
import os
from DraftParamParser import *  # Functions to process command line arguments for Draft.

# The argument name/types we're expecting from the command line arguments or Deadline.
expectedTypes = dict()
expectedTypes['inFile'] = '<string>'
expectedTypes['outFile'] = '<string>'
expectedTypes['frameList'] = '<string>'

# params now contains a dictionary of the parameters initialized to values from the command line arguments.
params = ParseCommandLine(expectedTypes, sys.argv)
inFilePattern = params['inFile']  # for example frame_name_###.ext, where ### represents a three digit frame number.
frames = FrameRangeToFrames(params['frameList'])  # Get a list of the individual frames we are to process
outFile = params['outFile']  # Where to store the transformed frames.

# create the pattern for the output file path and name from the output path and input pattern, if applicable.
if os.path.isdir(outFile):
    outPath = outFile  # We were given a path name without a file name.
    outPattern = os.path.basename(inFilePattern)  # Get just the file name from the input pattern.
    # Create the pattern for the output files from the directory path joined with the file name pattern.
    outFilePattern = os.path.join(outPath, outPattern)
else:
    outFilePattern = outFile

# Process each of the frames in the list of frames.
progressCounter = 0
for currFrame in frames:
    # Read in the frame.
    currFile = ReplaceFilenameHashesWithNumber(inFilePattern, currFrame)
    frame = Draft.Image.ReadFromFile(currFile)

    # Resize the frame to 640x480, fitting the width of the original image to 640.  Excess height will be cropped.  
    # If the resized image is shorter than 480 pixels, the excess background will be transparent.
    frame.Resize(640, 480, 'width', 'transparent')

    # Write the transformed frame.
    currOutFile = ReplaceFilenameHashesWithNumber(outFilePattern, currFrame)
    frame.WriteToFile(currOutFile)

    progressCounter = progressCounter + 1
    progress = progressCounter * 100 // len(frames)
    print("Progress: %i%%" % progress)
